<?php
class Configuration {
    static $instance = null;
    public static $baseDir;
    
    var $config;
    
    /*************************/
    /* Default config values */
    /*************************/
    
    /* General config */
	var $configgen = array(
		/* format: config key, default value */
		array("forumname", "Elf Lightweight forum"),
		array("adminpass", "21232f297a57a5a743894a0e4a801fc3"), // == 'admin'
        array("notifymail.topic", "ElfBB wants your attention"),
		array("notifymail.recipients", ""),
        array("notifymail.sender.email", "check@your.configuration"),
        array("notifymail.sender.name", "ElfBB Automatic"),
		array("hidenewposts", false),
		array("threadsperpage", 20),
		array("postsperpage", 20),
		array("dateformat", "d.m.Y H:i:s"),
		array("maxwordlen", 80),
		array("maxlenlastsender", 20),
		array("redirectdelay", 3000),
        array("composefieldsize", 10),
        array("cookielifetime", 30),
		array("showlangmenu", true),
		array("outputxhtml", false),
		array("showdebugmessages", true)
	);
	
	/* Directories */
	var $configdir = array(
		/* format: config key, default value */
		array("includedir", "/inc", ""),
		array("classdir", "/classes", ""),
		array("datadir", "/data", ""),
		array("langdir", "/lang", ""),
		array("tmpldir", "/tmpl", ""),
		array("themedir", "/themes", ""),
		array("webdir", "/web", "")
	);
	
	/* Database config */
	var $configdb = array(
		/* format: config key, default value */
		array("usedb", false),
		array("dbhost", ""),
		array("dbuname", ""),
		array("dbpassw", ""),
		array("dbname", ""),
	);
	
	/* Language config */
	var $configlang = array(
		/* format: config key, default value */
		array("defaultlang", "en"),
        array("languages", "en"),
        array("lang.en", "English"),
        array("lang.en.inmenu", true)
	);
	
	/***********/
	/* Methods */
	/***********/
    
	/* Static */
    static function getInstance() {
        if (! Configuration::$instance || Configuration::$instance == null) {
            Configuration::$instance = new Configuration();
        }
        
        return Configuration::$instance;
    }
	
	/* Instance */
	function Configuration() {
        if (isset(Configuration::$instance) && Configuration::$instance != null) {
            die("Configuration should only be instantiated via the getInstance() method");
        }
        
        $this->config = array();
        $this->setBasedir(dirname(__FILE__));
        $this->setDefaultValues();
    }
    
    function setBasedir($dir) {    	
    	if (strcmp("/", substr(strrev($dir), 0, 1)) == 0) {
    		$this->basedirectory = $dir;
    	} else {
    		$this->basedirectory = $dir . "/";
    	}        
    }
    
    function basedir() {      
        return $this->basedirectory;
    }
    
    function get($key) {
        return $this->config[$key];
    }
    
    function loadConfig($cfile) {
        //$this->config = array();

        if (! file_exists($cfile)) die($cfile ." not found");
        
        $cf = file($cfile);
        
        foreach ($cf as $row) {
            if ($row[0] == "/" || $row[0] == "*" || $row[0] == "<" || $row[0] == "?") { 
                continue; 
            }
            
            list($key, $value) = split("=", $row);
            
            $this->config[trim($key)] = trim($value);
        }
    }
	
	function setDefaultValues() {
		$this->resetGeneralCfg();		
		$this->resetDbCfg();
		$this->resetPathCfg();
        $this->resetLanguageCfg();
	}

	function resetGeneralCfg() {
		foreach ($this->configgen as $cfg) {
			if (! isset($this->config[$cfg[0]])) {
				$this->config[$cfg[0]] = $cfg[1];
			}
		}
	}
	
	function resetLanguageCfg() {
        foreach ($this->configlang as $cfg) {
			if (! isset($this->config[$cfg[0]])) {
				$this->config[$cfg[0]] = $cfg[1];
			}
		}
	}

	function resetDbCfg() {
		foreach ($this->configdb as $cfg) {
			if (! isset($this->config[$cfg[0]])) {
				$this->config[$cfg[0]] = $cfg[1];
			}
		}
	}
	
	function resetPathCfg() {
		foreach ($this->configdir as $cfg) {
			if (! isset($this->config[$cfg[0]])) {
				$this->config[$cfg[0]] = $cfg[1];
			}
		}
	}
	
	function saveNewValues() {
		$this->saveGenCfg();
		$this->savePathCfg();
		$this->saveDbCfg();
		$this->saveLanguageCfg();
	}
	
	function saveGenCfg() {
		foreach ($this->configgen as $cfg) {
			// a hack so passwords are md5 hashed
			if ($cfg[0] == 'adminpass' && isset($_POST['adminpass'])) {
				// check if the password has changed, if so, hash and store it					
				if (trim($_POST['adminpass']) != $this->config['adminpass'])  {
					$this->config['adminpass'] = md5(trim($_POST['adminpass']));
				}
			} else if (isset($_POST[str_replace(".", "_", $cfg[0])])) {
				$this->config[$cfg[0]] = $_POST[str_replace(".", "_", $cfg[0])];
			}
		}
	}
	
	function savePathCfg() {
		foreach ($this->configdir as $cfg) {
			if (isset($_POST[$cfg[0]])) {
				$this->config[$cfg[0]] = $_POST[$cfg[0]];
			}
		}
	}
	
	function saveDbCfg() {
		foreach ($this->configdb as $cfg) {
			if (isset($_POST[$cfg[0]])) {
				$this->config[$cfg[0]] = $_POST[$cfg[0]];
			}
		}
	} 
	
	function saveLanguageCfg() {
        // installed languages
        if (isset($_POST['languages'])) {
            $this->config['languages']   = $_POST['languages'];
            $this->config['defaultlang'] = $_POST['defaultlang'];
            
            foreach (split(",", $_POST['languages']) as $lng) {
                $this->config["lang.$lng"]        = $_POST["lang_{$lng}_name"];
                $this->config["lang.$lng.inmenu"] = $_POST["lang_{$lng}_inmenu"] ? true : false;
                
                // just checking. defaultlang must be present in menu
                if ($this->config['defaultlang'] == $lng) {
                    $this->config["lang.$lng.inmenu"] = true;
                }
            }
        }
        
        // install new languages
        if (isset($_POST['newlangs']) && strlen($_POST['newlangs']) > 0) {
            foreach (split(",", $_POST['newlangs']) as $lng) {
                if ($_POST["lang_{$lng}_install"] == 1) {
                    $this->config['languages']       .= ",". $lng;
                    $this->config["lang.$lng"]        = $_POST["lang_{$lng}_name"];
                    $this->config["lang.$lng.inmenu"] = true;
                }
            }
        }
	}
}
?>
