<?php
class BBCollection {
    static $instance = null;
    var    $threads; 

    function BBCollection() {
        if (isset(BBCollection::$instance) && BBCollection::$instance != null) {
            die("BBCollection should only be instantiated via the getInstance() method");
        }
        
        $this->threads = array();
    }
    
    /* BBCollection is a singleton class.
    getInstance should be used to get a BBCollection instance */
    static function getInstance() {
        if (BBCollection::$instance == null) {
            BBCollection::$instance = new BBCollection();
        }
        
        return BBCollection::$instance;
    }

    /* Load data from MySQL instead of XML files */
	function loadDataMysql() {
		$thread  = new dbconn();
		$comment = new dbconn();

		$thread->query("SELECT * FROM bbthread ORDER BY modified ASC, topic ASC");
		while ($item = $thread->next_assoc()) {
			$tmp = new BBItem();
			$tmp->parseItem($item);
			
			$comment->query("SELECT * FROM bbreply WHERE itemid = " . $tmp->id . " ORDER BY posted ASC");
			
			while ($cmnt = $comment->next_assoc()) {
				$tmp2 = new Comment();
				$tmp2->parseItem($cmnt);
				$tmp->addComment($tmp2);
			}
			
			if ($tmp->id > $this->threads["MaxId"]) {
				$this->threads["MaxId"] = $tmp->id;
			}
			
			$key = BBItem::getKeyFor($tmp->id);
			$this->threads["Items"][$key] = $tmp;
		}
	}

    /* Load data from XML files */
    function loadData() {
        $config = Configuration::getInstance();
        
    	// Load threads
    	$threads_xml = ($config->basedir() . $config->get('datadir') . "/threads.xml");
        BBItem::loadItems(&$this->threads["Items"], &$this->threads["MaxId"], $threads_xml);
		
        // Load comments
        $comments_xml = ($config->basedir() . $config->get('datadir') . "/comment.xml");
        $comments     = array();
        Comment::loadItems(&$comments["Items"], &$comments["MaxId"], $comments_xml);

		$this->threads["MaxIdC"]  = $comments["MaxId"];			
		$this->threads["NPublic"] = $this->getPublicThreadCnt();
        
		if (is_array($comments["Items"])) {
			// Attach comments to threads
			foreach ($comments["Items"] as $comment) {
				$key = BBItem::getKeyFor($comment->itemid);
				
				if (array_key_exists($key, $this->threads["Items"])) {
					$this->threads["Items"][$key]->addComment($comment);
				}
			}
		}
		
		if (is_array($this->threads["Items"])) {
        	reset($this->threads["Items"]);
		}
    }

    function sortItems($dir = "asc") {
		if (is_array($this->threads["Items"])) {
			uasort($this->threads["Items"], array("BBItem", "compare"));

			if (strcmp($dir, "desc") == 0) {
				$this->threads["Items"] = array_reverse($this->threads["Items"]);
			}
		}
    }

    function getNext() {
        if (is_array($this->threads["Items"])) {
			if ($item =& current($this->threads["Items"])) {
				next($this->threads["Items"]);
				return $item;
			} else {
				reset($this->threads["Items"]);
				return false;
			}
		} 
			
		return false;
    }
    
    function getNextPublic() {
		if (is_array($this->threads["Items"])) {
		    do {
		        $item =& current($this->threads["Items"]);
		        next($this->threads["Items"]);
		    } while ($item->public != true && $item !== false);
		    
			if ($item === false) {
				reset($this->threads["Items"]);
			} else {
			    return $item;
			}
		}
			
		return false;
    }
    
    function getThread($id) {
        $key = BBItem::getKeyFor($id);
        if (isset($this->threads["Items"][$key])) {
            return $this->threads["Items"][$key];
        }
        
        return false;
    }
    
	function addItem($item) {
	    $config = Configuration::getInstance();
	    
		if (! $config->get('usedb')) {
			$item->id = ++$this->threads["MaxId"];
			$key      = BBItem::getKeyFor($item->id);
			$this->threads["Items"][$key] = $item;
			
			return $item->id;
		} else {
			$dbc = new dbconn();
			$sql = sprintf(
				"INSERT INTO bbthread (public, posted, modified, topic, bread, ipaddr, creator) VALUES (%d, NOW(), NOW(), '%s', '%s', '%s', '%s')",
				$item->public, mysql_escape_string($item->topic), mysql_escape_string($item->bread), $item->ipaddr, mysql_escape_string($item->creator)
			);
			$id = $dbc->query($sql);
			
			return $id;
		}
	}
	
	function removeItem($item) {
		$config = Configuration::getInstance();
		
		if (! $config->get('usedb')) {
			$key = BBItem::getKeyFor($item->id);
        	if (isset($this->threads["Items"][$key])) {
        		unset($this->threads["Items"][$key]);
        		
        		return $this->saveThreads();
    		}
			
    		// Item not found (shouldn't happen)
    		return false;
		} else {
			$dbc = new dbconn();
			// Remove comments first
			$sql = sprintf("DELETE FROM bbreply WHERE itemid = %d", $item->id);
			$dbc->query($sql);
			
			// Remove thread
			$sql = sprintf("DELETE FROM bbthread WHERE id = %d", $item->id);
			$dbc->query($sql);
			
			return true;
		}
	}
	
	function removeItemComment($item, $comment) {
		$config = Configuration::getInstance();
		
		if (! $config->get('usedb')) {
			$key = BBItem::getKeyFor($item->id);
			if (isset($this->threads["Items"][$key])) {
        		$this->threads["Items"][$key]->removeComment($comment);
        		
        		return $this->saveThreads();
    		}
			
			return $this->saveThreads();
		} else {
			$dbc = new dbconn();
			
			// Remove comments first
			$sql = sprintf(
				"DELETE FROM bbreply WHERE id = %d",
				$comment->id
			);
			$dbc->query($sql);
			
			return true;
		}
	}
	
	function addItemComment($comment) {
	    $config = Configuration::getInstance();
	    
		if (! $config->get('usedb')) {
			$comment->id = ++$this->threads["MaxIdC"];
			$key         = BBItem::getKeyFor($comment->itemid);
			$this->threads["Items"][$key]->addComment($comment);
			
			return $comment->id;
		} else {
			$dbc = new dbconn();
			$sql = sprintf(
				"INSERT INTO bbreply (itemid, public, posted, bread, ipaddr, creator) VALUES (%d, %d, NOW(), '%s', '%s', '%s')",
				$comment->itemid, $comment->public, mysql_escape_string($comment->bread), $comment->ipaddr, mysql_escape_string($comment->creator)
			);
			$id = $dbc->query($sql);
			
			$sql = sprintf("UPDATE bbthread SET modified = NOW() WHERE id = %d", $comment->itemid);
			$dbc->query($sql);
			
			return $id;
		}
	}

    function getThreadCnt() {
        return is_array($this->threads["Items"]) ? sizeof($this->threads["Items"]) : 0;
    }
	
	function getPublicThreadCnt() {
        if (isset($this->threads["NPublic"])) {
            return $this->threads["NPublic"];
        }
    
		$pubCnt = 0;
		if (is_array($this->threads["Items"])) {
            foreach ($this->threads["Items"] as $item) {
                if ($item->public) {
                    $pubCnt++;
                }
            }
            reset($this->threads["Items"]);
		}
		return $pubCnt;
	}
	
	function saveThreads() {
	    $config      = Configuration::getInstance();
        $lockfile    = $config->basedir() . $config->get('datadir') . "/lockfile";
        $threadsxml  = $config->basedir() . $config->get('datadir') . "/threads.xml";
        $commentsxml = $config->basedir() . $config->get('datadir') . "/comment.xml";
        
        // Database interaction is (for now) left outside
        // so we do nothing here if DB is used for storing ElfBB
		if ($config->get('usedb')) {
			return true;
		}
	
		if ($lock = fopen($lockfile, "w")) {
			if (flock($lock, LOCK_EX)) {
				debugmessage("Got lock.");
			} else {
				debugmessage("Failed saving threads. Couldn't get lock.");
				return false;
			}
		} else {
			debugmessage("Failed saving threads. Couldn't get lock.");
			return false;
		}
	
		if (($fp = fopen($threadsxml, "w")) && ($fp2 = fopen($commentsxml, "w"))) {
			if ((flock($fp, LOCK_EX)) && (flock($fp2, LOCK_EX)))			 {
				fwrite($fp, "<xml version=\"1.0\">\n");
				fwrite($fp, "<root>\n");
			
				fwrite($fp2, "<xml version=\"1.0\">\n");
				fwrite($fp2, "<root>\n");
				
				foreach ($this->threads["Items"] as $item) {
					fwrite($fp, "\t<thread>\n");
					fwrite($fp, "\t\t<id>"        . $item->id      . "</id>\n");					
					fwrite($fp, "\t\t<public>"    . $item->public  . "</public>\n");
                    fwrite($fp, "\t\t<sticky>"    . $item->sticky  . "</sticky>\n");
					fwrite($fp, "\t\t<locked>"    . $item->locked  . "</locked>\n");
					fwrite($fp, "\t\t<posted>"    . $item->posted  . "</posted>\n");
					fwrite($fp, "\t\t<creator>"   . htmlspecialchars($item->creator) . "</creator>\n");
					fwrite($fp, "\t\t<ipaddr>"    . htmlspecialchars($item->ipaddr)  . "</ipaddr>\n");
					fwrite($fp, "\t\t<topic>"     . htmlspecialchars($item->topic)   . "</topic>\n");
					fwrite($fp, "\t\t<bread>"     . htmlspecialchars($item->bread)   . "</bread>\n");
					fwrite($fp, "\t</thread>\n");
					
					if ($item->getCommentCnt() > 0) {
						foreach($item->comments as $comment) {
							fwrite($fp2, "\t<comment>\n");
							fwrite($fp2, "\t\t<id>"      . $comment->id      . "</id>\n");					
							fwrite($fp2, "\t\t<itemid>"  . $comment->itemid  . "</itemid>\n");
							fwrite($fp2, "\t\t<public>"  . $comment->public  . "</public>\n");
							fwrite($fp2, "\t\t<posted>"  . $comment->posted  . "</posted>\n");
							fwrite($fp2, "\t\t<creator>" . htmlspecialchars($comment->creator) . "</creator>\n");
							fwrite($fp2, "\t\t<ipaddr>"  . htmlspecialchars($comment->ipaddr)  . "</ipaddr>\n");
							fwrite($fp2, "\t\t<bread>"   . htmlspecialchars($comment->bread)   . "</bread>\n");
							fwrite($fp2, "\t</comment>\n");
						}
					}
				}
				
				fwrite($fp, "</root>\n</xml>");
				fwrite($fp2, "</root>\n</xml>");
				flock($fp, LOCK_UN);
				flock($fp2, LOCK_UN);
			} else {
				debugmessage("Failed saving threads.");
                return false;
			}
			
			fclose($fp);
			fclose($fp2);
			
			debugmessage("Saved threads.");
            return true;
		} else {
			debugmessage("Failed saving threads.");
            return false;
		}
		
		flock($lock, LOCK_UN);
		fclose($lock);			
	}
}
?>
