<?php
class BBItem extends XmlItem {
    static $keystr = "bb";
    
    var $id;        // int
    var $public ;   // boolean (int)
    var $sticky;    // boolean (int)
	var $locked;    // boolean (int)
    var $posted;    // datetime (YYYY-MM-DD HH:MM:SS)
    var $modified;  // datetime (YYYY-MM-DD HH:MM:SS)
    var $topic;     // string
    var $bread;     // string
    var $ipaddr;    // string
	var $creator;   // string
    var $comments;  // Comment[]
    
    function BBItem() {
    	$this->posted = date("Y-m-d H:i:s", time());
    }

    function parseItem($xmldata) {
        $this->id       = $xmldata["id"];
        $this->public   = $xmldata["public"];
        $this->sticky   = $xmldata["sticky"];
		$this->locked   = $xmldata["locked"];
        $this->posted   = $xmldata["posted"];
        $this->creator  = stripslashes($xmldata["creator"]);
        $this->ipaddr   = $xmldata["ipaddr"];
        $this->topic    = stripslashes($xmldata["topic"]);
        $this->bread    = stripslashes($xmldata["bread"]);
		$this->modified = $xmldata["posted"];
		
		$this->comments = array(); // init
    }
    
    function getLastCommentLink() {
        $name = $this->creator;
        if (is_array($this->comments) && count($this->comments) > 0) {
            foreach ($this->comments as $comment); {
                if ($comment->public) {
                    $anchor = $comment->getAnchor();
                }
            }
        } else {
            $anchor = $this->getAnchor();
        }
        
        $u = new Url(Url::homeurl());
        $u->addparam("exp", $this->getId());
        
        return $u->tostring() . "#" . $anchor;
    }
    
    function getAnchor() {
    	return $this->id;
    }
    
    function toggleLocked() {
    	$this->locked = ! $this->locked;
    }
    
    function toggleSticky() {
    	$this->sticky = ! $this->sticky;
    }
    
    function togglePublic() {
    	$this->public = ! $this->public;
    }
    
    function addComment($comment) {
        $this->comments[] = $comment;
		
		if ($comment->public && (strtotime($comment->posted) > strtotime($this->modified))) {
			$this->modified = $comment->posted;
		}
		
        reset($this->comments);
    }
    
    function getId() {
        return $this->id;
    }

    function getComment($requested) {
        foreach ($this->comments as $item) {
            if ($item->getId() == $requested) {
                return $item;
            }
        }
        
        return false;
    }
    
    function removeComment($comment) {
    	foreach ($this->comments as $key => $item) {
            if ($item->getId() == $comment->getId()) {
            	unset($this->comments[$key]);
                
                return true;
            }
        }
        
        return false;
    }

    function getCommentCnt() {
        return is_array($this->comments) ? sizeof($this->comments) : 0;
    }
	
	function getPublicCommentCnt() {
		$pubCnt = 0;
		if (is_array($this->comments)) {
			for ($i = 0; $i < count($this->comments); $i++) {
				if ($this->comments[$i]->public == 1) {
					$pubCnt++;
				}
			}
		}
        return $pubCnt;
	}
    
    function getLastCommenter() {
        $name = $this->creator;
        if (is_array($this->comments)) {
            foreach ($this->comments as $comment); {
                if ($comment->public) {
                    $name = $comment->creator;
                }
            }
        }
        
        return $name;
    }
	
	function getLastCommentId() {
        $id = $this->id;
        if (is_array($this->comments)) {
            $id = $this->comments[count($this->comments) - 1]->id;
        }
        
        return $id;
    }

    function getNextComment() {
        $item = current($this->comments);
        
        if ($item !== false) {
            next($this->comments);
            
        	return $item;
        }
        
        return false;
    }
	
    function isLocked() {
        return $this->locked;
    }
    
    function isSticky() {
        return $this->sticky;
    }
    
    function isPublic() {
        return $this->public;
    }
    
	function contains(&$word) {
		if (! $this->public) {
			return false;
		}
		
		if (stripos($this->bread, $word) !== false) {
			return true;
		} else if (stripos($this->topic, $word) !== false) {
			return true;
		} else if (stripos($this->creator, $word) !== false) {
			return true;
		} else if (is_array($this->comments)) {
			foreach ($this->comments as $comment) {
				if ($comment->contains(&$word)) {
					return true;
				}
			}
		}
		
		return false;
	}

    /* Update modification timestamp */
	function touch() {
		$this->modified = date("Y-m-d H:i:s", time());
	}

    static function getKeyFor($id) {
        return BBItem::$keystr . $id;
    }

    /* Compare two BBItems (for sorting) */
    static function compare($a, $b) {
        if ($a->sticky && ! $b->sticky) {
            return 1;
        } elseif (! $a->sticky && $b->sticky) {
            return -1;
        } else {
            return strcmp($a->modified, $b->modified);
        }
    }

    /* Load items from XML */
    static function loadItems($itemarr, $maxid, $file) {
        $items = XmlItem::getItemsFromXml($file, "thread");
    
        if (! is_array($items) || sizeof($items) == 0) {
            return null;
        }

        foreach ($items as $item) {
            $tmp = new BBItem();
            $tmp->parseItem($item);
            $key = BBItem::getKeyFor($tmp->id);
            $itemarr[$key] = $tmp;
            
            if ($tmp->id > $maxid) {
                $maxid = $tmp->id;
            }
        }
    }
}
?>
